package Net::Netconf;
our $VERSION ='1.02';

=head1 NAME
Net::Netconf : Netconf Perl Client

=head1 SYNOPSIS
This API enables user to access Device remotely using Netconf Protocol and perform Operational and configuration tasks.



